@@ -117,7 +117,7 @@ namespace GmmLib
         // Mutex Object used for synchronization of ProcessSingleton Context
         static GMM_MUTEX_HANDLE           SingletonContextSyncMutex;
 #endif
-
+	GMM_PRIVATE_PAT PrivatePATTable[GMM_NUM_PAT_ENTRIES];
     public :
         //Constructors and destructors
         Context();
@@ -560,13 +560,24 @@ namespace GmmLib
     GMM_CACHE_POLICY* GMM_STDCALL CreateCachePolicyCommon();
     GMM_TEXTURE_CALC* GMM_STDCALL CreateTextureCalc(PLATFORM Platform, bool Override);
     GMM_PLATFORM_INFO_CLASS *GMM_STDCALL CreatePlatformInfo(PLATFORM Platform, bool Override);
-    
+
     private: 
         void GMM_STDCALL OverrideSkuWa();
+        
+    public:
+    /////////////////////////////////////////////////////////////////////////
+    /// Returns private PAT table array ptr
+    /// @return   PAT array ptr
+    /////////////////////////////////////////////////////////////////////////
+    GMM_INLINE GMM_PRIVATE_PAT *GMM_STDCALL GetPrivatePATTable()
+    {
+        return (&PrivatePATTable[0]);
+    }
+    
     };
 
 // Max number of Multi-Adapters allowed in the system
-#define MAX_NUM_ADAPTERS      32
+#define MAX_NUM_ADAPTERS      9
 //===========================================================================
 // typedef:
 //      _GMM_ADAPTER_INFO_
@@ -580,6 +591,8 @@ typedef struct _GMM_ADAPTER_INFO_
     int32_t             RefCount;                           // Ref Count for the number of Gmm UMD Lib process Singleton Context created per Process
     GMM_MUTEX_HANDLE    SyncMutex;                          // SyncMutex to protect access of Gmm UMD Lib process Singleton Context 
     ADAPTER_BDF         sBdf;                               // Adpater's Bus, Device and Function info for which Gmm UMD Lib process Singleton Context is created
+    _GMM_ADAPTER_INFO_  *pNext;                             // Linked List Next pointer to point to the Next Adapter node in the List
+
 }GMM_ADAPTER_INFO;
     
 ////////////////////////////////////////////////////////////////////////////////////
@@ -589,12 +602,15 @@ typedef struct _GMM_ADAPTER_INFO_
     class NON_PAGED_SECTION GmmMultiAdapterContext : public GmmMemAllocator
     {
     private:
-        GMM_ADAPTER_INFO                AdapterInfo[MAX_NUM_ADAPTERS];
+        GMM_ADAPTER_INFO                AdapterInfo[MAX_NUM_ADAPTERS];// For Static Initialization of adapter.
         GMM_MUTEX_HANDLE                MAContextSyncMutex;         // SyncMutex to protect access of GmmMultiAdpaterContext
         uint32_t                        NumAdapters;
 	void*                           pCpuReserveBase;
 	uint64_t                        CpuReserveSize;
-
+        GMM_ADAPTER_INFO                *pHeadNode;// For dynamic Initialization of adapter.
+                                                   // The Multi-Adapter Initialization is done dynamiclly using a Linked list Vector
+                                                   // pHeadNode points to the root node of the linked list and registers the first
+                                                   // adapter received from UMD.
     public:
         //Constructors and destructors
         GmmMultiAdapterContext();
@@ -614,6 +630,7 @@ typedef struct _GMM_ADAPTER_INFO_
         int32_t GMM_STDCALL             DecrementRefCount(ADAPTER_BDF sBdf);
         GMM_STATUS GMM_STDCALL          LockSingletonContextSyncMutex(ADAPTER_BDF sBdf);
         GMM_STATUS GMM_STDCALL          UnlockSingletonContextSyncMutex(ADAPTER_BDF sBdf);
+        void *GMM_STDCALL               GetAdapterNode(ADAPTER_BDF sBdf); // Replacemet for GetAdapterIndex, now get adapter info from a node in the linked list
     }; // GmmMultiAdapterContext
 
 } //namespace
